// TODO: Support generics
// TODO: Support inject/binding parameters
// TODO: Throw errors/warnings on incorrect annotation
var s = require('typescript-schema');
function toRawRef(cls) {
    return {
        typeKind: null,
        module: cls.module || cls.container.name,
        name: cls.name
    };
}
function generateWiring(schema) {
    var unwired = {
        injections: {},
        bindings: {}
    };
    s.schemaVisitor(schema, {
        onClass: function (classSchema) {
            if (classSchema.decorators) {
                var bindingDec = classSchema.decorators.filter(function (decorator) {
                    return decorator.decorator === 'bind';
                })[0];
                if (bindingDec) {
                    var type;
                    if (classSchema.extends) {
                        type = toRawRef(classSchema.extends);
                    }
                    else if (classSchema.implements.length === 1) {
                        type = toRawRef(classSchema.implements[0]);
                    }
                    else {
                    }
                    var binding = {
                        type: toRawRef(classSchema)
                    };
                    unwired.bindings[type.module + '.' + type.name] = binding;
                }
            }
            if (classSchema.constructorSchema) {
                var cs = classSchema.constructorSchema;
                cs.parameters.forEach(function (parameter, index) {
                    if (parameter.decorators) {
                        var injectDec = parameter.decorators.filter(function (decorator) {
                            return decorator.decorator === 'inject';
                        })[0];
                        if (injectDec) {
                            var type;
                            if (parameter.type.typeKind === s.TypeKind.CLASS) {
                                type = toRawRef(parameter.type);
                            }
                            else if (parameter.type.typeKind === s.TypeKind.INTERFACE) {
                                type = toRawRef(parameter.type);
                            }
                            else {
                                type = toRawRef(parameter.type);
                            }
                            var injection = {
                                type: type,
                                index: index,
                                name: parameter.name
                            };
                            unwired.injections[classSchema.container.name + '.' + classSchema.name] = [injection];
                        }
                    }
                });
            }
        }
    });
    return unwired;
}
exports.generateWiring = generateWiring;
//# sourceMappingURL=modelGenerator.js.map